@@ -21,28 +21,37 @@
 #include "palm_sync.h"
 #include "palm_format.h"
 
+#include <opensync/opensync-data.h>
+#include <opensync/opensync-format.h>
+#include <opensync/opensync-plugin.h>
+#include <opensync/opensync-context.h>
+
 static OSyncChange *psyncContactCreate(PSyncEnv *env, PSyncEntry *entry, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, entry, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, env, entry, error);
 	PSyncDatabase *db = entry->db;
 
-	OSyncChange *change = osync_change_new();
+	OSyncChange *change = osync_change_new(error);
 	if (!change)
 		goto error;
-	osync_change_set_member(change, env->member);
 
+	OSyncData *data = osync_data_new(NULL, 0, env->contact_format, error);
+	if (!data)
+		goto error_free_change;
+	
+	osync_change_set_data(change, data);
+	osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_ADDED);
+		
 	char *uid = g_strdup_printf("uid-AddressDB-%ld", entry->id);
 	osync_change_set_uid(change, uid);
 	g_free(uid);
 	
-	osync_change_set_objformat_string(change, "palm-contact");
-	
 	if ((entry->attr &  dlpRecAttrDeleted) || (entry->attr & dlpRecAttrArchived)) {
 		if ((entry->attr & dlpRecAttrArchived)) {
 			osync_trace(TRACE_INTERNAL, "Archieved");
 		}
 		//we have a deleted record
-		osync_change_set_changetype(change, CHANGE_DELETED);
+		osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_DELETED);
 	} else {
 		/* Create the object data */
 		PSyncContactEntry *contact = osync_try_malloc0(sizeof(PSyncContactEntry), error);
@@ -60,12 +69,10 @@ static OSyncChange *psyncContactCreate(PSyncEnv *env, PSyncEntry *entry, OSyncEr
 	    if (catname)
 			contact->categories = g_list_append(contact->categories, g_strdup(catname));
 		
-		osync_change_set_data(change, (void *)contact, sizeof(PSyncContactEntry), TRUE);
+		osync_data_set_data(data, (void *)contact, sizeof(PSyncContactEntry));
 		
 		if (entry->attr & dlpRecAttrDirty)  {
-			osync_change_set_changetype(change, CHANGE_MODIFIED);
-		} else {
-			osync_change_set_changetype(change, CHANGE_UNKNOWN);
+			osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_MODIFIED);
 		}
 	}
 	
@@ -73,42 +80,53 @@ static OSyncChange *psyncContactCreate(PSyncEnv *env, PSyncEntry *entry, OSyncEr
 	return change;
 
 error_free_change:
-	osync_change_free(change);
+	osync_change_unref(change);
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return NULL;
 }
 
-osync_bool psyncContactGetChangeInfo(OSyncContext *ctx, OSyncError **error)
+static void psyncContactGetChanges(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, ctx, error);
-	PSyncEnv *env = (PSyncEnv *)osync_context_get_plugin_data(ctx);
+	OSyncError *error = NULL;
+	PSyncEnv *env = (PSyncEnv *)data;
 	PSyncDatabase *db = NULL;
 	PSyncEntry *entry = NULL;
 	
-	if (!(db = psyncDBOpen(env, "AddressDB", error)))
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
+	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
+	
+	osync_trace(TRACE_INTERNAL, "Opening conduit");
+	dlp_OpenConduit(env->socket);
+	
+	if (!(db = psyncDBOpen(env, "AddressDB", &error)))
 		goto error;
 	
-	if (osync_member_get_slow_sync(env->member, "contact") == TRUE) {
+	if (osync_objtype_sink_get_slowsync(sink) == TRUE) {
 		osync_trace(TRACE_INTERNAL, "slow sync");
 		
 		int n;
-		for (n = 0; (entry = psyncDBGetNthEntry(db, n, error)); n++) {
+		for (n = 0; (entry = psyncDBGetNthEntry(db, n, &error)); n++) {
 			osync_trace(TRACE_INTERNAL, "Got record with id %ld", entry->id);
 			
-			OSyncChange *change = psyncContactCreate(env, entry, error);
+			if ((entry->attr &  dlpRecAttrDeleted) || (entry->attr & dlpRecAttrArchived))
+				continue;
+
+			OSyncChange *change = psyncContactCreate(env, entry, &error);
 			if (!change)
 				goto error;
 			
 			if (osync_change_get_data(change) == NULL)
 				continue;
 
-			osync_change_set_changetype(change, CHANGE_ADDED);
+			/* It's a slow-sync - mark everything as changetype ADDED */
+			osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_ADDED);
+
 			osync_context_report_change(ctx, change);
 		}
 	} else {
-		while ((entry = psyncDBGetNextModified(db, error))) {
-			OSyncChange *change = psyncContactCreate(env, entry, error);
+		while ((entry = psyncDBGetNextModified(db, &error))) {
+			OSyncChange *change = psyncContactCreate(env, entry, &error);
 			if (!change)
 				goto error;
 			
@@ -116,19 +134,22 @@ osync_bool psyncContactGetChangeInfo(OSyncContext *ctx, OSyncError **error)
 		}
 	}
 				
-	if (osync_error_is_set(error))
+	if (osync_error_is_set(&error))
 		goto error_close_db;
 	
 	psyncDBClose(db);
 	
+	osync_context_report_success(ctx);
+	
 	osync_trace(TRACE_EXIT, "%s", __func__);
-	return TRUE;
+	return;
 
 error_close_db:
 	psyncDBClose(db);
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return FALSE;
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error) ? osync_error_print(&error) : "nil");
+	osync_context_report_osyncerror(ctx, error);
+	osync_error_unref(&error);
 }
 
 /*
@@ -147,24 +168,29 @@ error:
 			}
 		} else {*/
 
-osync_bool psyncContactCommit(OSyncContext *ctx, OSyncChange *change)
+
+static void psyncContactCommit(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, ctx, change);
-	PSyncEnv *env = (PSyncEnv *)osync_context_get_plugin_data(ctx);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, data, info, ctx, change);
+	PSyncEnv *env = (PSyncEnv *)data;
 	PSyncDatabase *db = NULL;
 	PSyncEntry *entry = NULL;
-	PSyncContactEntry *contact = NULL;
 	OSyncError *error = NULL;
 	unsigned long id = 0;
 
+	PSyncContactEntry *contact = osync_try_malloc0(sizeof(PSyncContactEntry), &error); 
+	if (!contact)
+		goto error;
+
 	//open the DB
 	if (!(db = psyncDBOpen(env, "AddressDB", &error)))
 		goto error;
 	
-	contact = (PSyncContactEntry *)osync_change_get_data(change);
+	OSyncData *change_data = osync_change_get_data(change);
+	osync_data_get_data(change_data, (char **) &contact, NULL);
 			
 	switch (osync_change_get_changetype(change)) {
-		case CHANGE_MODIFIED:
+		case OSYNC_CHANGE_TYPE_MODIFIED:
 			//Get the id
 			id = psyncUidGetID(osync_change_get_uid(change), &error);
 			if (!id)
@@ -206,7 +232,7 @@ osync_bool psyncContactCommit(OSyncContext *ctx, OSyncChange *change)
 				goto error;
 				
 			break;
-		case CHANGE_ADDED:
+		case OSYNC_CHANGE_TYPE_ADDED:
 			//Add a new entry
 			osync_trace(TRACE_INTERNAL, "Find category");
 			
@@ -217,7 +243,7 @@ osync_bool psyncContactCommit(OSyncContext *ctx, OSyncChange *change)
 			
 			GList *c = NULL;
 			for (c = contact->categories; c; c = c->next) {
-				osync_trace(TRACE_INTERNAL, "searching category %s\n", c->data);
+				osync_trace(TRACE_INTERNAL, "searching category %s\n", c->data ? c->data : "nil");
 				entry->category = psyncDBCategoryToId(db, c->data, NULL);
 				if (entry->category != 0) {
 					osync_trace(TRACE_INTERNAL, "Found category %i\n", entry->category);
@@ -242,7 +268,7 @@ osync_bool psyncContactCommit(OSyncContext *ctx, OSyncChange *change)
 			osync_change_set_uid(change, uid);
 			g_free(uid);
 			break;
-		case CHANGE_DELETED:
+		case OSYNC_CHANGE_TYPE_DELETED:
 			id = psyncUidGetID(osync_change_get_uid(change), &error);
 			if (!id)
 				goto error;
@@ -258,11 +284,63 @@ osync_bool psyncContactCommit(OSyncContext *ctx, OSyncChange *change)
 	
 	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
-	return TRUE;
+	return;
 
 error:
-	osync_context_report_osyncerror(ctx, &error);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-	return FALSE;
+	osync_context_report_osyncerror(ctx, error);
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error) ? osync_error_print(&error) : "nil");
+	osync_error_unref(&error);
 }
 
+static void psyncContactSyncDone(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
+	PSyncEnv *env = (PSyncEnv *)data;
+
+	PSyncDatabase *db = NULL;
+	OSyncError *error = NULL;
+
+	if ((db = psyncDBOpen(env, "AddressDB", &error))) {
+		osync_trace(TRACE_INTERNAL, "Reseting Sync Flags for AddressDB");
+		dlp_ResetSyncFlags(env->socket, db->handle);
+		dlp_CleanUpDatabase(env->socket, db->handle);
+		psyncDBClose(db);
+	} else {
+		osync_context_report_osyncerror(ctx, error);
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error) ? osync_error_print(&error) : "nil");
+		osync_error_unref(&error);
+		return;
+	}
+
+	osync_context_report_success(ctx);
+	
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return;
+}
+
+osync_bool psyncContactInitialize(PSyncEnv *env, OSyncPluginInfo *info, OSyncError **error)
+{
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
+	env->contact_format = osync_format_env_find_objformat(formatenv, "palm-contact");
+	
+	env->contact_sink = osync_objtype_sink_new("contact", error);
+	if (!env->contact_sink)
+		return FALSE;
+	
+	osync_objtype_sink_add_objformat(env->contact_sink, "palm-contact");
+	
+	OSyncObjTypeSinkFunctions functions;
+	memset(&functions, 0, sizeof(functions));
+	functions.connect= psyncConnect;
+	functions.disconnect= psyncDisconnect;
+	functions.get_changes = psyncContactGetChanges;
+	functions.commit = psyncContactCommit;
+	functions.sync_done = psyncContactSyncDone;
+	
+	/* We pass the OSyncFileDir object to the sink, so we dont have to look it up
+	 * again once the functions are called */
+	osync_objtype_sink_set_functions(env->contact_sink, functions, NULL);
+	osync_plugin_info_add_objtype(info, env->contact_sink);
+	
+	return TRUE;
+}
