@@ -21,10 +21,10 @@
 #include "palm_sync.h"
 #include "palm_format.h"
 
-#include "palm_todo.h"
-#include "palm_contact.h"
-#include "palm_event.h"
-#include "palm_note.h"
+#include <opensync/opensync-data.h>
+#include <opensync/opensync-format.h>
+#include <opensync/opensync-plugin.h>
+#include <opensync/opensync-context.h>
 
 #define CATCOUNT 16
 
@@ -85,7 +85,7 @@ void psyncDBClose(PSyncDatabase *db)
 
 PSyncDatabase *psyncDBOpen(PSyncEnv *env, char *name, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, env, name, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, env, name ? name : "nil", error);
 	
 	struct DBInfo dbInfo;
 	memset(&dbInfo, 0, sizeof(struct DBInfo));
@@ -120,7 +120,7 @@ PSyncDatabase *psyncDBOpen(PSyncEnv *env, char *name, OSyncError **error)
 	//open it
 	int ret = dlp_OpenDB(env->socket, 0, dlpOpenReadWrite, name, &(db->handle));
 	if (_psyncCheckReturn(env->socket, ret, error) != PSYNC_NO_ERROR) {
-		osync_error_update(error, "Unable to open %s: %s", name, osync_error_print(error));
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to open %s: %s", name, osync_error_print(error));
 		goto error_free_db;
 	}
 
@@ -151,7 +151,7 @@ error_free_db:
 #endif
 	g_free(db);
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return NULL;
 }
 
@@ -181,10 +181,10 @@ PSyncEntry *psyncDBGetEntryByID(PSyncDatabase *db, unsigned long id, OSyncError
 #endif
 	PSyncError err = _psyncCheckReturn(db->env->socket, ret, error);
 	if (err == PSYNC_ERROR_OTHER) {
-		osync_error_update(error, "Unable to get next entry: %s", osync_error_print(error));
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get next entry: %s", osync_error_print(error));
 		goto error_free_buffer;
 	} else if (err == PSYNC_ERROR_NOT_FOUND) {
-		osync_error_free(error);
+		osync_error_unref(error);
 		goto error_free_buffer;
 	}
 	
@@ -199,7 +199,7 @@ error_free_entry:
 	g_free(entry);
 error:
 	if (osync_error_is_set(error))
-		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	else
 		osync_trace(TRACE_EXIT, "%s: Not Found", __func__);
 	return NULL;
@@ -231,10 +231,10 @@ PSyncEntry *psyncDBGetNthEntry(PSyncDatabase *db, int nth, OSyncError **error)
 #endif
 	PSyncError err = _psyncCheckReturn(db->env->socket, ret, error);
 	if (err == PSYNC_ERROR_OTHER) {
-		osync_error_update(error, "Unable to get next entry: %s", osync_error_print(error));
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get next entry: %s", osync_error_print(error));
 		goto error_free_buffer;
 	} else if (err == PSYNC_ERROR_NOT_FOUND) {
-		osync_error_free(error);
+		osync_error_unref(error);
 		goto error_free_buffer;
 	}
 	
@@ -249,7 +249,7 @@ error_free_entry:
 	g_free(entry);
 error:
 	if (osync_error_is_set(error))
-		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	else
 		osync_trace(TRACE_EXIT, "%s: Not Found", __func__);
 	return NULL;
@@ -282,10 +282,10 @@ PSyncEntry *psyncDBGetNextModified(PSyncDatabase *db, OSyncError **error)
 
 	PSyncError err = _psyncCheckReturn(db->env->socket, ret, error);
 	if (err == PSYNC_ERROR_OTHER) {
-		osync_error_update(error, "Unable to get next entry: %s", osync_error_print(error));
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get next entry: %s", osync_error_print(error));
 		goto error_free_buffer;
 	} else if (err == PSYNC_ERROR_NOT_FOUND) {
-		osync_error_free(error);
+		osync_error_unref(error);
 		goto error_free_buffer;
 	}
 	
@@ -300,7 +300,7 @@ error_free_entry:
 	g_free(entry);
 error:
 	if (osync_error_is_set(error))
-		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	else
 		osync_trace(TRACE_EXIT, "%s: Not Found", __func__);
 	return NULL;
@@ -312,7 +312,7 @@ osync_bool psyncDBDelete(PSyncDatabase *db, int id, OSyncError **error)
 	
 	int ret = dlp_DeleteRecord(db->env->socket, db->handle, 0,  id);
 	if (_psyncCheckReturn(db->env->socket, ret, error) != PSYNC_NO_ERROR) {
-		osync_error_update(error, "Unable to delete file: %s", osync_error_print(error));
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to delete file: %s", osync_error_print(error));
 		goto error;
 	}
 	
@@ -320,7 +320,7 @@ osync_bool psyncDBDelete(PSyncDatabase *db, int id, OSyncError **error)
 	return TRUE;
 
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return FALSE;
 }
 
@@ -338,7 +338,7 @@ osync_bool psyncDBWrite(PSyncDatabase *db, PSyncEntry *entry, OSyncError **error
 	int ret = dlp_WriteRecord(db->env->socket, db->handle, entry->attr, entry->id, entry->category, entry->buffer, entry->size, 0);
 #endif
 	if (_psyncCheckReturn(db->env->socket, ret, error) != PSYNC_NO_ERROR) {
-		osync_error_update(error, "Unable to write file: %s", osync_error_print(error));
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to write file: %s", osync_error_print(error));
 		goto error;
 	}
 	
@@ -346,7 +346,7 @@ osync_bool psyncDBWrite(PSyncDatabase *db, PSyncEntry *entry, OSyncError **error
 	return TRUE;
 
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return FALSE;
 }
 
@@ -371,7 +371,7 @@ osync_bool psyncDBAdd(PSyncDatabase *db, PSyncEntry *entry, unsigned long *id, O
 	ret = dlp_WriteRecord(db->env->socket, db->handle, 0, 0, entry->category, ((pi_buffer_t*)entry->buffer)->data, ((pi_buffer_t*)entry->buffer)->used, id);
 #endif
 	if (_psyncCheckReturn(db->env->socket, ret, error) != PSYNC_NO_ERROR) {
-		osync_error_update(error, "Unable to add file: %s", osync_error_print(error));
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to add file: %s", osync_error_print(error));
 		goto error;
 	}
 	
@@ -379,7 +379,7 @@ osync_bool psyncDBAdd(PSyncDatabase *db, PSyncEntry *entry, unsigned long *id, O
 	return TRUE;
 
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return FALSE;
 }
 
@@ -394,22 +394,22 @@ const char *psyncDBCategoryFromId(PSyncDatabase *db, int id, OSyncError **error)
 		
 	const char *ret = db->cai.name[id];
 
-	osync_trace(TRACE_EXIT, "%s: %s", __func__, ret);
+	osync_trace(TRACE_EXIT, "%s: %s", __func__, ret ? ret : "nil");
 	return ret;
 
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return NULL;
 }
 
 int psyncDBCategoryToId(PSyncDatabase *db, const char *name, OSyncError **error)
 {	
-	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, db, name, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, db, name ? name : "nil", error);
 	
 	int i = 0;
 	for (i = 0; i < CATCOUNT; i++) {
 		if (db->cai.name[i][0] != '\0') {
-			osync_trace(TRACE_INTERNAL, "remote: cat %d [%s] ID %d renamed %d", i, db->cai.name[i], db->cai.ID[i], db->cai.renamed[i]);
+			osync_trace(TRACE_INTERNAL, "remote: cat %d [%s] ID %d renamed %d", i, db->cai.name[i] ? db->cai.name[i] : "nil", db->cai.ID[i], db->cai.renamed[i]);
 			if (!strcmp(db->cai.name[i], name)) {
 				osync_trace(TRACE_EXIT, "%s: %i", __func__, i);
 				return i;
@@ -438,9 +438,9 @@ static osync_bool _connectDevice(PSyncEnv *env, unsigned int timeout, OSyncError
 	}
 
 	if (env->conntype != PILOT_DEVICE_NETWORK) {
-		rate_buf = g_strdup_printf("PILOTRATE=%i", env->speed);
+		rate_buf = g_strdup_printf("%i", env->speed);
 		osync_trace(TRACE_INTERNAL, "setting PILOTRATE=%i", env->speed);
-		putenv(rate_buf);
+		setenv("PILOTRATE", rate_buf, TRUE);
 		g_free(rate_buf);
 	}
 
@@ -495,30 +495,28 @@ static osync_bool _connectDevice(PSyncEnv *env, unsigned int timeout, OSyncError
 error_free_listen:
 	pi_close(listen_sd);
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return FALSE;
 }
 
 static gboolean _psyncPoll(gpointer data)
 {
-	gboolean poll_still_required = TRUE;
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, data);
 	PSyncEnv *env = (PSyncEnv *)data;
 	
 	if (env->socket > 0) {
 		osync_trace(TRACE_EXIT, "%s: Already have a socket", __func__);
-		return FALSE;
+		return TRUE;
 	}
 
-	OSyncError *error = NULL;
-	if (_connectDevice(env, 1, &error)) {
-		osync_member_request_synchronization(env->member);
-		poll_still_required = FALSE;
-	} else
-		osync_error_free(&error);
+	//OSyncError *error = NULL;
+	//if (_connectDevice(env, 1, &error))
+		//osync_member_request_synchronization(env->member);
+	//else
+		//osync_error_unref(&error);
 		
 	osync_trace(TRACE_EXIT, "%s", __func__);
-	return poll_still_required;
+	return TRUE;
 }
 
 static gboolean _psyncPing(gpointer data)
@@ -540,28 +538,27 @@ static gboolean _psyncPing(gpointer data)
 	return TRUE;
 }
 
-static void psyncThreadStart(PSyncEnv *env)
+static void psyncThreadStart(PSyncEnv *env, OSyncPluginInfo *info)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, env);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, env, info);
 	
-	GMainContext *context = osync_member_get_loop(env->member);
+	GMainContext *context = osync_plugin_info_get_loop(info);
 	
 	GSource *source = g_timeout_source_new(5000);
 	g_source_set_callback(source, _psyncPing, env, NULL);
-	env->sourceTags->ping = g_source_attach(source, context);
-	g_source_unref (source);
-
+	g_source_attach(source, context);
+	
+	//FIXME also polls at the end of a hot sync ... 
 	source = g_timeout_source_new(1000);
 	g_source_set_callback(source, _psyncPoll, env, NULL);
-	env->sourceTags->poll = g_source_attach(source, context);
-	g_source_unref (source);
+	g_source_attach(source, context);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static osync_bool psyncSettingsParse(PSyncEnv *env, const char *config, unsigned int size, OSyncError **error)
+static osync_bool psyncSettingsParse(PSyncEnv *env, const char *config, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, env, config, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, env, config ? config : "nil", error);
 	xmlDoc *doc = NULL;
 	xmlNode *cur = NULL;
 
@@ -576,7 +573,7 @@ static osync_bool psyncSettingsParse(PSyncEnv *env, const char *config, unsigned
 	env->mismatch = 1;
 	env->popup = 0;
 
-	doc = xmlParseMemory(config, size);
+	doc = xmlParseMemory(config, strlen(config));
 
 	if (!doc) {
 		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to parse settings");
@@ -640,78 +637,48 @@ static osync_bool psyncSettingsParse(PSyncEnv *env, const char *config, unsigned
 error_free_doc:
 		xmlFreeDoc(doc);
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return FALSE;
 }
 
-static void *psyncInitialize(OSyncMember *member, OSyncError **error)
+unsigned long psyncUidGetID(const char *uid, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, member, error);
-
-	PSyncEnv *env = osync_try_malloc0(sizeof(PSyncEnv), error);
-	if (!env)
-		goto error;
-	env->sourceTags = NULL;
-	env->sourceTags = osync_try_malloc0(sizeof(PSyncSourceTags), error);
-	if (!env)
-		goto error;
-		
-	char *configdata = NULL;
-	int configsize = 0;
-	if (!osync_member_get_config(member, &configdata, &configsize, error)) {
-		osync_error_update(error, "Unable to get config data: %s", osync_error_print(error));
-		goto error_free_env;
+	unsigned long id = 0;
+	if (sscanf(uid, "uid-%*[^-]-%ld", &id) != 1) {
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to parse uid %s", uid);
+		return 0;
 	}
+	osync_trace(TRACE_INTERNAL, "Got id %ld", id);
+	if (!id)
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Found 0 ID");
 	
-	if (!psyncSettingsParse(env, configdata, configsize, error))
-		goto error_free_config;
-	
-	env->member = member;
-	g_free(configdata);
-	
-	psyncThreadStart(env);
-	
-	osync_trace(TRACE_EXIT, "%s, %p", __func__, env);
-	return (void *)env;
-
-error_free_config:
-	g_free(configdata);
-error_free_env:
-	if (env->sourceTags)
-		g_free(env->sourceTags);
-	g_free(env);
-error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return NULL;
+	return id;
 }
 
-static void psyncConnect(OSyncContext *ctx)
+void psyncConnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
-	PSyncEnv *env = (PSyncEnv *)osync_context_get_plugin_data(ctx);
 	OSyncError *error = NULL;
+	
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
+	PSyncEnv *env = (PSyncEnv *)data;
 	int ret;
-	struct SysInfo sys_info;
-	char anchor[25];
 
 	//now connect with the palm
 	if (!_connectDevice(env, env->timeout, &error))
 		goto error;
 
-	// Do the following so that OpenSync will work with password enabled
-        // Palm devices (OS 4 and higher).
-        if (dlp_ReadSysInfo(env->socket, &sys_info) < 0) {
-                osync_trace(TRACE_INTERNAL, "Unable to read SysInfo");
-        }
-
 	//check the user
 	ret = dlp_ReadUserInfo(env->socket, &env->user);
+	osync_trace(TRACE_INTERNAL, "test %i", ret);
 	if (_psyncCheckReturn(env->socket, ret, &error) != PSYNC_NO_ERROR) {
-		osync_error_update(&error, "Unable to read UserInfo: %s", osync_error_print(&error));
+		//osync_error_update(&error, "Unable to read UserInfo: %s", osync_error_print(&error));
 		goto error;
 	}
 	
-	osync_trace(TRACE_SENSITIVE, "User: %s, %i\n", env->user.username, env->user.userID);
+	if (env->user.userID == 0)
+		strcpy(env->user.username, "");
+		
+	osync_trace(TRACE_INTERNAL, "User: %s, %i\n", env->user.username ? env->user.username : "nil", env->user.userID);
 	/*if (strcmp(User.username, conn->username) || User.userID != conn->id) {
 		//Id or username mismatch
 		switch (conn->mismatch) {
@@ -738,13 +705,9 @@ static void psyncConnect(OSyncContext *ctx)
 	}*/
 	
 	if (env->user.lastSyncPC == 0) {
-		//Device has been reset
+		//Device has been reseted
 		osync_trace(TRACE_INTERNAL, "Detected that the Device has been reset");
-		osync_member_set_slow_sync(env->member, "data", TRUE);
-	} else {
-	  snprintf(anchor, 24, "%li", env->user.lastSyncPC);
-	  if (!osync_anchor_compare(env->member, "lastSyncPC", anchor))
-	    osync_member_set_slow_sync(env->member, "data", TRUE);
+		//osync_member_set_slow_sync(env->member, "data", TRUE);
 	}
 	
 	osync_context_report_success(ctx);
@@ -759,130 +722,21 @@ error:
 		env->socket = 0;
 	}
 	
-	osync_context_report_osyncerror(ctx, &error);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
+	osync_context_report_osyncerror(ctx, error);
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error) ? osync_error_print(&error) : "nil");
 }
 
-unsigned long psyncUidGetID(const char *uid, OSyncError **error)
+void psyncSyncDone(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
-	unsigned long id = 0;
-	if (sscanf(uid, "uid-%*[^-]-%ld", &id) != 1) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to parse uid %s", uid);
-		return 0;
-	}
-	osync_trace(TRACE_INTERNAL, "Got id %ld", id);
-	if (!id)
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Found 0 ID");
-	
-	return id;
-}
-
-static void psyncGetChangeinfo(OSyncContext *ctx)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
-	PSyncEnv *env = (PSyncEnv *)osync_context_get_plugin_data(ctx);
-	OSyncError *error = NULL;
-
-	osync_trace(TRACE_INTERNAL, "Opening conduit");
-	dlp_OpenConduit(env->socket);
-
-#ifdef HAVE_TODO	
-	if (osync_member_objtype_enabled(env->member, "todo") && !psyncTodoGetChangeInfo(ctx, &error))
-		goto error;
-#endif	
-	
-#ifdef HAVE_CONTACT	
-	if (osync_member_objtype_enabled(env->member, "contact") && !psyncContactGetChangeInfo(ctx, &error))
-		goto error;
-#endif	
-		
-#ifdef HAVE_EVENT	
-	if (osync_member_objtype_enabled(env->member, "event") && !psyncEventGetChangeInfo(ctx, &error))
-		goto error;
-#endif	
-
-#ifdef HAVE_NOTE	
-	if (osync_member_objtype_enabled(env->member, "note") && !psyncNoteGetChangeInfo(ctx, &error))
-		goto error;
-#endif	
-
-	osync_context_report_success(ctx);
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return;
-
-error:
-	osync_context_report_osyncerror(ctx, &error);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-}
-
-static void psyncDone(OSyncContext *ctx)
-{
-	char *anchor;
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
-	PSyncEnv *env = (PSyncEnv *)osync_context_get_plugin_data(ctx);
-
-	PSyncDatabase *db = NULL;
-	OSyncError *error = NULL;
-
-	osync_trace(TRACE_INTERNAL, "Removing Palm sync timer, ping: %i",
-		    g_source_remove(env->sourceTags->ping));
-	//Just in case it's still running
-	osync_trace(TRACE_INTERNAL, "Removing Palm sync timer, poll: %i",
-		    g_source_remove(env->sourceTags->poll));
-
-	if ((db = psyncDBOpen(env, "AddressDB", &error))) {
-		osync_trace(TRACE_INTERNAL, "Reseting Sync Flags for AddressDB");
-		dlp_ResetSyncFlags(env->socket, db->handle);
-		dlp_CleanUpDatabase(env->socket, db->handle);
-		psyncDBClose(db);
-	}
-	
-	if ((db = psyncDBOpen(env, "ToDoDB", &error))) {
-		osync_trace(TRACE_INTERNAL, "Reseting Sync Flags for ToDoDB");
-		dlp_ResetSyncFlags(env->socket, db->handle);
-		dlp_CleanUpDatabase(env->socket, db->handle);
-		psyncDBClose(db);
-	}
-	
-	if ((db = psyncDBOpen(env, "DatebookDB", &error))) {
-		osync_trace(TRACE_INTERNAL, "Reseting Sync Flags for DatebookDB");
-		dlp_ResetSyncFlags(env->socket, db->handle);
-		dlp_CleanUpDatabase(env->socket, db->handle);
-		psyncDBClose(db);
-	}
-
-	if ((db = psyncDBOpen(env, "MemoDB", &error))) {
-		osync_trace(TRACE_INTERNAL, "Reseting Sync Flags for MemoDB");
-		dlp_ResetSyncFlags(env->socket, db->handle);
-		dlp_CleanUpDatabase(env->socket, db->handle);
-		psyncDBClose(db);
-	}
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
+	PSyncEnv *env = (PSyncEnv *)data;
 
 	//Set the log and sync entries on the palm
-	dlp_AddSyncLogEntry(env->socket, "Sync Successful\n");
+	dlp_AddSyncLogEntry(env->socket, "Sync Successfull\n");
 	dlp_AddSyncLogEntry(env->socket, "Thank you for using\n");
 	dlp_AddSyncLogEntry(env->socket, "OpenSync");
 
-	
-	anchor = osync_anchor_retrieve(env->member, "lastSyncPC");
-	if ((anchor == NULL) || (env->user.lastSyncPC == 0)) {
-	  srandom(time(NULL));
-	  // Stolen from jpilot (jpilot.org), complete with comment:
-	  /* RAND_MAX is 32768 on Solaris machines for some reason.
-	   * If someone knows how to fix this, let me know.
-	   */
-	  if (RAND_MAX==32768) {
-	    env->user.lastSyncPC = 1+(2000000000.0*random()/(2147483647+1.0));
-	  } else {
-	    env->user.lastSyncPC = 1+(2000000000.0*random()/(RAND_MAX+1.0));
-	  }
-	  anchor = malloc(25);
-	  snprintf(anchor, 24, "%li", env->user.lastSyncPC);
-	  osync_trace(TRACE_INTERNAL, "Made a new lastSyncPC of %s", anchor);
-	  osync_anchor_update(env->member, "lastSyncPC", anchor);
-	  free(anchor);
-	}
-
+	env->user.lastSyncPC = 1;
 	env->user.lastSyncDate = time(NULL);
 	env->user.successfulSyncDate = time(NULL);
 	
@@ -892,13 +746,12 @@ static void psyncDone(OSyncContext *ctx)
 
 	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
-	return;
 }
 
-static void psyncDisconnect(OSyncContext *ctx)
+void psyncDisconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
-	PSyncEnv *env = (PSyncEnv *)osync_context_get_plugin_data(ctx);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
+	PSyncEnv *env = (PSyncEnv *)data;
 
 	if (env->currentDB)
 		psyncDBClose(env->currentDB);
@@ -911,61 +764,112 @@ static void psyncDisconnect(OSyncContext *ctx)
 		env->socket = 0;
 	}
 
+	unsetenv("PILOTRATE");
+
 	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return;
 }
 
+static void *psyncInitialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError **error)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, info, error);
+
+	PSyncEnv *env = osync_try_malloc0(sizeof(PSyncEnv), error);
+	if (!env)
+		goto error;
+		
+	if (!psyncSettingsParse(env, osync_plugin_info_get_config(info), error))
+		goto error_free_env;
+	
+	if (!psyncContactInitialize(env, info, error))
+		goto error_free_env;
+	
+	/* Add the main sink */
+	//OSyncObjTypeSink *sink = osync_objtype_sink_new(NULL, error);
+	//if (!sink)
+	//	goto error_free_env;
+	
+	/* All sinks have the same functions of course */
+	//OSyncObjTypeSinkFunctions functions;
+	//memset(&functions, 0, sizeof(functions));
+	//functions.connect = psyncConnect;
+	//functions.disconnect = psyncDisconnect;
+	//functions.sync_done = psyncSyncDone;
+	
+	//osync_objtype_sink_set_functions(sink, functions, NULL);
+	//osync_plugin_info_add_objtype(info, sink);
+
+	psyncThreadStart(env, info);
+	
+	osync_trace(TRACE_EXIT, "%s, %p", __func__, env);
+	return (void *)env;
+
+error_free_env:
+	g_free(env);
+error:
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
+	return NULL;
+}
+
 static void psyncFinalize(void *data)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, data);
 	PSyncEnv *env = (PSyncEnv *)data;
 
-	if (env->sourceTags)
-		g_free(env->sourceTags);
+	if (env->username)
+		g_free(env->username);
+
+	if (env->sockaddr)
+		g_free(env->sockaddr);
+
+	if (env->codepage)
+		g_free(env->codepage);
+
+	osync_objtype_sink_unref(env->contact_sink);
+
 	g_free(env);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-void get_info(OSyncEnv *env)
+static osync_bool psyncDiscover(void *data, OSyncPluginInfo *info, OSyncError **error)
 {
-	OSyncPluginInfo *info = osync_plugin_new_info(env);
-	info->name = "palm-sync";
-	info->longname = "Palm Synchronization Plugin";
-	info->description = "Plugin to synchronize Palm devices";
-	info->version = 1;
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, error);
+	PSyncEnv *env = (PSyncEnv *)data;
 	
-	info->functions.initialize = psyncInitialize;
-	info->functions.connect = psyncConnect;
-	info->functions.sync_done = psyncDone;
-	info->functions.disconnect = psyncDisconnect;
-	info->functions.finalize = psyncFinalize;
-	info->functions.get_changeinfo = psyncGetChangeinfo;
-
-#ifdef HAVE_CONTACT	
-	osync_plugin_accept_objtype(info, "contact");
-	osync_plugin_accept_objformat(info, "contact", "palm-contact", NULL);
-	osync_plugin_set_commit_objformat(info, "contact", "palm-contact", psyncContactCommit);
-#endif	
+	osync_objtype_sink_set_available(env->contact_sink, TRUE);
 	
-#ifdef HAVE_TODO	
-	osync_plugin_accept_objtype(info, "todo");
-	osync_plugin_accept_objformat(info, "todo", "palm-todo", NULL);
-	osync_plugin_set_commit_objformat(info, "todo", "palm-todo", psyncTodoCommit);
-#endif	
-	
-#ifdef HAVE_EVENT	
-	osync_plugin_accept_objtype(info, "event");
-	osync_plugin_accept_objformat(info, "event", "palm-event", NULL);
-	osync_plugin_set_commit_objformat(info, "event", "palm-event", psyncEventCommit);
-#endif	
-	
-#ifdef HAVE_NOTE	
-	osync_plugin_accept_objtype(info, "note");
-	osync_plugin_accept_objformat(info, "note", "palm-note", NULL);
-	osync_plugin_set_commit_objformat(info, "note", "palm-note", psyncNoteCommit);
-	osync_plugin_set_read_objformat(info, "note", "palm-note", psyncNoteRead);
-#endif	
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
 }
 
+osync_bool get_sync_info(OSyncPluginEnv *env, OSyncError **error)
+{
+	OSyncPlugin *plugin = osync_plugin_new(error);
+	if (!plugin)
+		goto error;
+	
+	osync_plugin_set_name(plugin, "palm-sync");
+	osync_plugin_set_longname(plugin, "Palm Synchronization Plugin");
+	osync_plugin_set_description(plugin, "Plugin to synchronize Palm devices");
+	
+	osync_plugin_set_initialize(plugin, psyncInitialize);
+	osync_plugin_set_finalize(plugin, psyncFinalize);
+	osync_plugin_set_discover(plugin, psyncDiscover);
+	
+	osync_plugin_env_register_plugin(env, plugin);
+	osync_plugin_unref(plugin);
+	
+	return TRUE;
+	
+error:
+	osync_trace(TRACE_ERROR, "Unable to register: %s", osync_error_print(error) ? osync_error_print(error) : "nil");
+	osync_error_unref(error);
+	return FALSE;
+}
+
+int get_version(void)
+{
+	return 1;
+}
