@@ -48,8 +48,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
-
 #define PILOT_DEVICE_SERIAL 0
 #define PILOT_DEVICE_USB_VISOR 1
 #define PILOT_DEVICE_IRDA 2
@@ -89,14 +87,7 @@ typedef struct PSyncEntry {
 	int index;
 } PSyncEntry;
 
-typedef struct PSyncSourceTags {
-	guint ping;
-	guint poll;
-} PSyncSourceTags;
-
 struct PSyncEnv {
-	OSyncMember *member;
-	
 	char *username;
 	int id;
 	char *sockaddr;
@@ -112,8 +103,9 @@ struct PSyncEnv {
 	struct PilotUser user;
 	
 	char *codepage;
-
-	PSyncSourceTags *sourceTags;
+	
+	OSyncObjFormat *contact_format;
+	OSyncObjTypeSink *contact_sink;
 };
 
 void psyncDBClose(PSyncDatabase *db);
@@ -129,4 +121,8 @@ osync_bool psyncDBAdd(PSyncDatabase *db, PSyncEntry *entry, unsigned long *id, O
 osync_bool psyncDBDelete(PSyncDatabase *db, int id, OSyncError **error);
 PSyncEntry *psyncDBGetEntryByID(PSyncDatabase *db, unsigned long id, OSyncError **error);
 		
+void psyncConnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx);
+void psyncDisconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx);
+void psyncSyncDone(void *data, OSyncPluginInfo *info, OSyncContext *ctx);
+
 #endif //_PALM_SYNC_H
