@@ -21,7 +21,7 @@
 #include "palm_sync.h"
 #include "palm_format.h"
 
-static OSyncChange *psyncTodoCreate(PSyncEnv *env, PSyncEntry *entry, OSyncError **error)
+static OSyncChange *psyncTodoCreate(PSyncEntry *entry, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, entry, error);
 	PSyncDatabase *db = entry->db;
@@ -29,7 +29,6 @@ static OSyncChange *psyncTodoCreate(PSyncEnv *env, PSyncEntry *entry, OSyncError
 	OSyncChange *change = osync_change_new();
 	if (!change)
 		goto error;
-	osync_change_set_member(change, env->member);
 	
 	/* Set the uid */
 	char *uid = g_strdup_printf("uid-ToDoDB-%ld", entry->id);
@@ -80,7 +79,7 @@ static OSyncChange *psyncTodoCreate(PSyncEnv *env, PSyncEntry *entry, OSyncError
 error_free_change:
 	osync_change_free(change);
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return NULL;
 }
 
@@ -107,7 +106,7 @@ osync_bool psyncTodoGetChangeInfo(OSyncContext *ctx, OSyncError **error)
 
 			osync_trace(TRACE_INTERNAL, "Got all recored with id %ld", entry->id);
 			
-			OSyncChange *change = psyncTodoCreate(env, entry, error);
+			OSyncChange *change = psyncTodoCreate(entry, error);
 			if (!change)
 				goto error;
 			
@@ -125,7 +124,7 @@ osync_bool psyncTodoGetChangeInfo(OSyncContext *ctx, OSyncError **error)
 			if (entry == NULL)
 				continue;
 
-			OSyncChange *change = psyncTodoCreate(env, entry, error);
+			OSyncChange *change = psyncTodoCreate(entry, error);
 			if (!change)
 				goto error;
 			
@@ -144,7 +143,7 @@ osync_bool psyncTodoGetChangeInfo(OSyncContext *ctx, OSyncError **error)
 error_close_db:
 	psyncDBClose(db);
 error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error) ? osync_error_print(error) : "nil");
 	return FALSE;
 }
 
@@ -202,7 +201,7 @@ osync_bool psyncTodoCommit(OSyncContext *ctx, OSyncChange *change)
 			
 			GList *c = NULL;
 			for (c = todo->categories; c; c = c->next) {
-				osync_trace(TRACE_INTERNAL, "searching category %s\n", c->data);
+				osync_trace(TRACE_INTERNAL, "searching category %s\n", c->data ? c->data : "nil");
 				entry->category = psyncDBCategoryToId(db, c->data, NULL);
 				if (entry->category != 0) {
 					osync_trace(TRACE_INTERNAL, "Found category %i\n", entry->category);
@@ -252,7 +251,7 @@ osync_bool psyncTodoCommit(OSyncContext *ctx, OSyncChange *change)
 
 error:
 	osync_context_report_osyncerror(ctx, &error);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error) ? osync_error_print(&error) : "nil");
 	return FALSE;
 }
 
